@@ -1,3 +1,4 @@
+{-# LANGUAGE CPP #-}
 -- |
 -- Module      : Data.Vector.Fusion.Util
 -- Copyright   : (c) Roman Leshchinskiy 2009
@@ -16,7 +17,9 @@ module Data.Vector.Fusion.Util (
   delay_inline, delayed_min
 ) where
 
+#if !MIN_VERSION_base(4,8,0)
 import Control.Applicative (Applicative(..))
+#endif
 
 -- | Identity monad
 newtype Id a = Id { unId :: a }
@@ -29,7 +32,7 @@ instance Applicative Id where
   Id f <*> Id x = Id (f x)
 
 instance Monad Id where
-  return     = Id
+  return = pure
   Id x >>= f = f x
 
 -- | Box monad
@@ -43,7 +46,7 @@ instance Applicative Box where
   Box f <*> Box x = Box (f x)
 
 instance Monad Box where
-  return      = Box
+  return = pure
   Box x >>= f = f x
 
 -- | Delay inlining a function until late in the game (simplifier phase 0).
@@ -55,4 +58,3 @@ delay_inline f = f
 delayed_min :: Int -> Int -> Int
 {-# INLINE [0] delayed_min #-}
 delayed_min m n = min m n
-
