@@ -102,7 +102,7 @@ module Data.Vector.Generic (
   takeWhile, dropWhile,
 
   -- ** Partitioning
-  partition, unstablePartition, span, break,
+  partition, partitionWith, unstablePartition, span, break,
 
   -- ** Searching
   elem, notElem, find, findIndex, findIndices, elemIndex, elemIndices,
@@ -165,7 +165,7 @@ module Data.Vector.Generic (
   liftShowsPrec, liftReadsPrec,
 
   -- ** @Data@ and @Typeable@
-  gfoldl, dataCast, mkType
+  gfoldl, gunfold, dataCast, mkVecType, mkVecConstr, mkType
 ) where
 
 import           Data.Vector.Generic.Base
@@ -211,17 +211,20 @@ import Data.Typeable ( Typeable1, gcast1 )
 
 #include "vector.h"
 
-import Data.Data ( Data, DataType )
+import Data.Data ( Data, DataType, Constr, Fixity(Prefix),
+                   mkDataType, mkConstr, constrIndex,
 #if MIN_VERSION_base(4,2,0)
-import Data.Data ( mkNoRepType )
+                   mkNoRepType )
 #else
-import Data.Data ( mkNorepType )
+                   mkNorepType )
+#endif
+import qualified Data.Traversable as T (Traversable(mapM))
+
+#if !MIN_VERSION_base(4,2,0)
 mkNoRepType :: String -> DataType
 mkNoRepType = mkNorepType
 #endif
 
-import qualified Data.Traversable as T (Traversable(mapM))
-
 -- Length information
 -- ------------------
 
@@ -472,10 +475,12 @@ unsafeDrop :: Vector v a => Int -> v a -> v a
 {-# INLINE unsafeDrop #-}
 unsafeDrop n v = unsafeSlice n (length v - n) v
 
-{-# RULES
 
-"slice/new [Vector]" forall i n p.
-  slice i n (new p) = new (New.slice i n p)
+-- Turned off due to: https://github.com/haskell/vector/issues/257
+-- "slice/new [Vector]" forall i n p.
+--   slice i n (new p) = new (New.slice i n p)
+
+{-# RULES
 
 "init/new [Vector]" forall p.
   init (new p) = new (New.init p)
@@ -573,7 +578,7 @@ unfoldrNM n f = unstreamM . MBundle.unfoldrNM n f
 -- | /O(n)/ Construct a vector with @n@ elements by repeatedly applying the
 -- generator function to the already constructed part of the vector.
 --
--- > constructN 3 f = let a = f <> ; b = f <a> ; c = f <a,b> in f <a,b,c>
+-- > constructN 3 f = let a = f <> ; b = f <a> ; c = f <a,b> in <a,b,c>
 --
 constructN :: forall v a. Vector v a => Int -> (v a -> a) -> v a
 {-# INLINE constructN #-}
@@ -602,7 +607,7 @@ constructN !n f = runST (
 -- repeatedly applying the generator function to the already constructed part
 -- of the vector.
 --
--- > constructrN 3 f = let a = f <> ; b = f<a> ; c = f <b,a> in f <c,b,a>
+-- > constructrN 3 f = let a = f <> ; b = f<a> ; c = f <b,a> in <c,b,a>
 --
 constructrN :: forall v a. Vector v a => Int -> (v a -> a) -> v a
 {-# INLINE constructrN #-}
@@ -1375,6 +1380,19 @@ partition_stream f s = s `seq` runST (
     v2 <- unsafeFreeze mv2
     return (v1,v2))
 
+partitionWith :: (Vector v a, Vector v b, Vector v c) => (a -> Either b c) -> v a -> (v b, v c)
+{-# INLINE partitionWith #-}
+partitionWith f = partition_with_stream f . stream
+
+partition_with_stream :: (Vector v a, Vector v b, Vector v c) => (a -> Either b c) -> Bundle u a -> (v b, v c)
+{-# INLINE_FUSED partition_with_stream #-}
+partition_with_stream f s = s `seq` runST (
+  do
+    (mv1,mv2) <- M.partitionWithBundle f s
+    v1 <- unsafeFreeze mv1
+    v2 <- unsafeFreeze mv2
+    return (v1,v2))
+
 -- | /O(n)/ Split the vector in two parts, the first one containing those
 -- elements that satisfy the predicate and the second one those that don't.
 -- The order of the elements is not preserved but the operation is often
@@ -2192,10 +2210,26 @@ gfoldl :: (Vector v a, Data a)
 {-# INLINE gfoldl #-}
 gfoldl f z v = z fromList `f` toList v
 
+mkVecConstr :: String -> Constr
+{-# INLINE mkVecConstr #-}
+mkVecConstr name = mkConstr (mkVecType name) "fromList" [] Prefix
+
+mkVecType :: String -> DataType
+{-# INLINE mkVecType #-}
+mkVecType name = mkDataType name [mkVecConstr name]
+
 mkType :: String -> DataType
 {-# INLINE mkType #-}
 mkType = mkNoRepType
 
+gunfold :: (Vector v a, Data a)
+        => (forall b r. Data b => c (b -> r) -> c r)
+        -> (forall r. r -> c r)
+        -> Constr -> c (v a)
+gunfold k z c = case constrIndex c of
+  1 -> k (z fromList)
+  _ -> error "gunfold"
+
 #if __GLASGOW_HASKELL__ >= 707
 dataCast :: (Vector v a, Data a, Typeable v, Typeable t)
 #else
