@@ -1,5 +1,7 @@
 {-# LANGUAGE CPP, DeriveDataTypeable, FlexibleInstances, MagicHash, MultiParamTypeClasses, ScopedTypeVariables #-}
 
+
+
 -- |
 -- Module      : Data.Vector.Storable.Mutable
 -- Copyright   : (c) Roman Leshchinskiy 2009-2010
@@ -51,13 +53,18 @@ module Data.Vector.Storable.Mutable(
   -- * Unsafe conversions
   unsafeCast,
 
+
   -- * Raw pointers
   unsafeFromForeignPtr, unsafeFromForeignPtr0,
   unsafeToForeignPtr,   unsafeToForeignPtr0,
   unsafeWith
 ) where
 
-import Control.DeepSeq ( NFData(rnf) )
+import Control.DeepSeq ( NFData(rnf)
+#if MIN_VERSION_deepseq(1,4,3)
+                       , NFData1(liftRnf)
+#endif
+                       )
 
 import qualified Data.Vector.Generic.Mutable as G
 import Data.Vector.Storable.Internal
@@ -76,7 +83,7 @@ import GHC.ForeignPtr
 
 import GHC.Base ( Int(..) )
 
-import Foreign.Ptr
+import Foreign.Ptr (castPtr,plusPtr)
 import Foreign.Marshal.Array ( advancePtr, copyArray, moveArray )
 
 import Control.Monad.Primitive
@@ -91,10 +98,13 @@ import Prelude hiding ( length, null, replicate, reverse, map, read,
 
 import Data.Typeable ( Typeable )
 
+
 -- Data.Vector.Internal.Check is not needed
 #define NOT_VECTOR_MODULE
 #include "vector.h"
 
+
+
 -- | Mutable 'Storable'-based vectors
 data MVector s a = MVector {-# UNPACK #-} !Int
                            {-# UNPACK #-} !(ForeignPtr a)
@@ -106,6 +116,11 @@ type STVector s = MVector s
 instance NFData (MVector s a) where
   rnf (MVector _ _) = ()
 
+#if MIN_VERSION_deepseq(1,4,3)
+instance NFData1 (MVector s) where
+  liftRnf _ (MVector _ _) = ()
+#endif
+
 instance Storable a => G.MVector MVector a where
   {-# INLINE basicLength #-}
   basicLength (MVector n _) = n
@@ -130,7 +145,7 @@ instance Storable a => G.MVector MVector a where
         fp <- mallocVector n
         return $ MVector n fp
     where
-      size = sizeOf (undefined :: a)
+      size = sizeOf (undefined :: a) `max` 1
       mx = maxBound `quot` size :: Int
 
   {-# INLINE basicInitialize #-}
@@ -201,8 +216,7 @@ storableSetAsPrim n fp x _y = withForeignPtr fp $ \ ptr  -> do
     poke ptr x
      -- we dont equate storable and prim reps, so we need to write to a slot
      -- in storable
-     -- then read it back as a prim type
-     -- then use prim memset
+     -- then read it back as a prim
     w<- peakPrimPtr_vector ((castPtr ptr) :: Ptr  b) 0
     memsetPrimPtr_vector ((castPtr ptr) `plusPtr` sizeOf x ) (n-1)  w
 
@@ -210,8 +224,7 @@ storableSetAsPrim n fp x _y = withForeignPtr fp $ \ ptr  -> do
 
 {-
 AFTER primitive 0.7 is pretty old, move to using setPtr. which is really
-a confusing misnomer for whats often called memset (initialize an array with a
-default value)
+a confusing misnomer for whats often called memset (intialize )
 -}
 -- Fill a memory block with the given value. The length is in
 -- elements of type @a@ rather than in bytes.
@@ -269,8 +282,13 @@ null = G.null
 -- Extracting subvectors
 -- ---------------------
 
--- | Yield a part of the mutable vector without copying it.
-slice :: Storable a => Int -> Int -> MVector s a -> MVector s a
+-- | Yield a part of the mutable vector without copying it. The vector must
+-- contain at least @i+n@ elements.
+slice :: Storable a
+      => Int  -- ^ @i@ starting index
+      -> Int  -- ^ @n@ length
+      -> MVector s a
+      -> MVector s a
 {-# INLINE slice #-}
 slice = G.slice
 
@@ -467,7 +485,9 @@ unsafeCopy = G.unsafeCopy
 -- copied to a temporary vector and then the temporary vector was copied
 -- to the target vector.
 move :: (PrimMonad m, Storable a)
-     => MVector (PrimState m) a -> MVector (PrimState m) a -> m ()
+     => MVector (PrimState m) a   -- ^ target
+     -> MVector (PrimState m) a   -- ^ source
+     -> m ()
 {-# INLINE move #-}
 move = G.move
 
