@@ -4,10 +4,11 @@ import qualified Data.Vector.Unboxed as V
 
 import System.Random.MWC
 import Control.Monad.ST ( runST )
+import Data.Word
 
-randomVector :: (Variate a, V.Unbox a) => Int -> IO (V.Vector a)
-randomVector n = withSystemRandom $ \g ->
-  do
+randomVector :: (Variate a, V.Unbox a) => Word32 -> Int -> IO (V.Vector a)
+randomVector seed n = do
+    g <- initialize (V.singleton seed)
     xs <- sequence $ replicate n $ uniform g
     io (return $ V.fromListN n xs)
   where
